<?php
/**
 * @author Rafael Ostertag <rafael.ostertag@math.uzh.ch>
 */

//namespace qfq;

use qfq\Store;

//use qfq;

require_once(__DIR__ . '/../../qfq/database/Database.php');
require_once(__DIR__ . '/../../qfq/store/Store.php');
require_once(__DIR__ . '/../../qfq/store/Sip.php');

/**
 * Class AbstractDatabaseTest
 */
abstract class AbstractDatabaseTest extends PHPUnit_Framework_TestCase {

    /**
     * @var null
     */
    static protected $mysqli = null;

    /**
     * @var qfq\Database
     */
    protected $db = null;

    /*
     * @var qfq\Store
     */
    protected $store = null;

    /*
     * @var qfq\Sip
     */
    protected $sip = null;

    /**
     * @param $filename
     * @param $ignoreError
     *
     * @throws Exception
     */
    protected function executeSQLFile($filename, $ignoreError) {

        $sqlStatements = file_get_contents($filename);
        if (!self::$mysqli->multi_query($sqlStatements) && !$ignoreError) {
            throw new Exception("mysqli->multi_query() failed: " . AbstractDatabaseTest::$mysqli->error);
        }

        // discard all results
        do {
            if ($res = AbstractDatabaseTest::$mysqli->store_result()) {
                $res->free();
            }
        } while (AbstractDatabaseTest::$mysqli->more_results() && AbstractDatabaseTest::$mysqli->next_result());
    }

    /**
     * @throws Exception
     */
    protected function setUp() {

        // Init the store also reads db credential configuration
        $this->store = qfq\Store::getInstance('', true);

        $this->sip = new qfq\Sip('fakesessionname', true);
        $this->sip->sipUniqId('badcaffee1234');

        // SWITCH to TestDB
        $this->store->setVar(SYSTEM_DB_NAME, $this->store->getVar(SYSTEM_DB_NAME_TEST, STORE_SYSTEM), STORE_SYSTEM);

        if ($this->db === null) {
            $this->db = new qfq\Database();
        }

        /// Establish additional mysqli access
        $dbserver = $this->store->getVar(SYSTEM_DB_SERVER, STORE_SYSTEM);
        $dbuser = $this->store->getVar(SYSTEM_DB_USER, STORE_SYSTEM);
        $db = $this->store->getVar(SYSTEM_DB_NAME, STORE_SYSTEM);
        $dbpw = $this->store->getVar(SYSTEM_DB_PASSWORD, STORE_SYSTEM);

        if (self::$mysqli === null) {

            self::$mysqli = new \mysqli($dbserver, $dbuser, $dbpw, $db);

            if (self::$mysqli->connect_errno) {
                throw new Exception("Unable to connect to mysql server");
            }
        }

    }
}
